/*
 * LCD44780.cpp
 *
 *  Created on: Aug 21, 2012
 *      Author: jgnoss
 */
#include <inttypes.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <alloca.h>
#include "util/delay.h"
#include "avr/eeprom.h"
#include <avr/pgmspace.h>
#include "LCD44780.h"


LCD44780::LCD44780(
		e_LCD_Type LCD_Type,
		volatile uint8_t &port_d4,
		uint8_t pin_d4,
		volatile uint8_t &port_d5,
		uint8_t pin_d5,
		volatile uint8_t &port_d6,
		uint8_t pin_d6,
		volatile uint8_t &port_d7,
		uint8_t pin_d7,
		volatile uint8_t &port_rs,
		uint8_t pin_rs,
		volatile uint8_t &port_e1,
		uint8_t pin_e1,
		volatile uint8_t &port_e2,
		uint8_t pin_e2) :
		LCD_State(LCD_State_Unknown),
		LCD_Type(LCD_Type),
		iZeile(0),
		iSpalte(0),
		port_e1(port_e1),
		pin_e1(pin_e1),
		port_e2(port_e2),
		pin_e2(pin_e2),
		port_rs(port_rs),
		pin_rs(pin_rs),
		port_d4(port_d4),
		pin_d4(pin_d4), port_d5(port_d5),
		pin_d5(pin_d5),
		port_d6(port_d6),
		pin_d6(pin_d6), port_d7(port_d7),
		pin_d7(pin_d7)
{
	switch (LCD_Type) {
	case LCD_Type_1x8:
		nZeilen = 1;
		nSpalten = 8;
		map_Linestart_DDRAM[0] = 0x80 + 0x00;
		map_Linestart_DDRAM[1] = 0x80 + 0x00;
		map_Linestart_DDRAM[2] = 0x80 + 0x00;
		map_Linestart_DDRAM[3] = 0x80 + 0x00;
		break;

	case LCD_Type_1x16_MUX8:
		// Anm.: die Zeichen liegen bei 0x00 bis 0x0F
		nZeilen = 1;
		nSpalten = 16;
		map_Linestart_DDRAM[0] = 0x80 + 0x00;
		map_Linestart_DDRAM[1] = 0x80 + 0x00;
		map_Linestart_DDRAM[2] = 0x80 + 0x00;
		map_Linestart_DDRAM[3] = 0x80 + 0x00;
		break;

	case LCD_Type_1x16_MUX16:
		// Anm. die ersten 8 Zeichen liegen bei 0x00 bis 0x07, die
		// rechten 8 bei 0x40 bis 0x47.
		// Deshalb Sonderbehandlung noetig; ausserdem kein Schreiben von
		// Strings ueber diese Grenze hinweg moeglich!
		nZeilen = 1;
		nSpalten = 16;
		map_Linestart_DDRAM[0] = 0x80 + 0x00;
		map_Linestart_DDRAM[1] = 0x80 + 0x00;
		map_Linestart_DDRAM[2] = 0x80 + 0x00;
		map_Linestart_DDRAM[3] = 0x80 + 0x00;
		break;

	case LCD_Type_1x20:
		nZeilen = 1;
		nSpalten = 20;
		map_Linestart_DDRAM[0] = 0x80 + 0x00;
		map_Linestart_DDRAM[1] = 0x80 + 0x00;
		map_Linestart_DDRAM[2] = 0x80 + 0x00;
		map_Linestart_DDRAM[3] = 0x80 + 0x00;
		break;

	case LCD_Type_1x40:
		nZeilen = 1;
		nSpalten = 40;
		map_Linestart_DDRAM[0] = 0x80 + 0x00;
		map_Linestart_DDRAM[1] = 0x80 + 0x00;
		map_Linestart_DDRAM[2] = 0x80 + 0x00;
		map_Linestart_DDRAM[3] = 0x80 + 0x00;
		break;

	case LCD_Type_2x8:
		nZeilen = 2;
		nSpalten = 8;
		map_Linestart_DDRAM[0] = 0x80 + 0x00;
		map_Linestart_DDRAM[1] = 0x80 + 0x00;
		map_Linestart_DDRAM[2] = 0x80 + 0x00;
		map_Linestart_DDRAM[3] = 0x80 + 0x00;
		break;

	case LCD_Type_2x12:
		nZeilen = 2;
		nSpalten = 12;
		map_Linestart_DDRAM[0] = 0x80 + 0x00;
		map_Linestart_DDRAM[1] = 0x80 + 0x40;
		map_Linestart_DDRAM[2] = 0x80 + 0x00;
		map_Linestart_DDRAM[3] = 0x80 + 0x00;
		break;

	case LCD_Type_2x16:
		nZeilen = 2;
		nSpalten = 16;
		map_Linestart_DDRAM[0] = 0x80 + 0x00;
		map_Linestart_DDRAM[1] = 0x80 + 0x40;
		map_Linestart_DDRAM[2] = 0x80 + 0x00;
		map_Linestart_DDRAM[3] = 0x80 + 0x00;
		break;

	case LCD_Type_2x20:
		nZeilen = 2;
		nSpalten = 20;
		map_Linestart_DDRAM[0] = 0x80 + 0x00;
		map_Linestart_DDRAM[1] = 0x80 + 0x40;
		map_Linestart_DDRAM[2] = 0x80 + 0x00;
		map_Linestart_DDRAM[3] = 0x80 + 0x00;
		break;

	case LCD_Type_2x24:
		nZeilen = 2;
		nSpalten = 24;
		map_Linestart_DDRAM[0] = 0x80 + 0x00;
		map_Linestart_DDRAM[1] = 0x80 + 0x40;
		map_Linestart_DDRAM[2] = 0x80 + 0x00;
		map_Linestart_DDRAM[3] = 0x80 + 0x00;
		break;

	case LCD_Type_2x40:
		nZeilen = 2;
		nSpalten = 40;
		map_Linestart_DDRAM[0] = 0x80 + 0x00;
		map_Linestart_DDRAM[1] = 0x80 + 0x40;
		map_Linestart_DDRAM[2] = 0x80 + 0x00;
		map_Linestart_DDRAM[3] = 0x80 + 0x00;
		break;

	case LCD_Type_4x16:
		nZeilen = 4;
		nSpalten = 16;
		map_Linestart_DDRAM[0] = 0x80 + 0x00;
		map_Linestart_DDRAM[1] = 0x80 + 0x40;
		map_Linestart_DDRAM[2] = 0x80 + 0x10;
		map_Linestart_DDRAM[3] = 0x80 + 0x50;
		break;

	case LCD_Type_4x20:

		// TODO:
		// EA DIP 204-6 (KS0073): Adressen 0x20 0x40 0x60 0x80
		// beim Initialisieren noch dazu schicken: 0b00001001 (mit RE=1).

		nZeilen = 4;
		nSpalten = 20;
		map_Linestart_DDRAM[0] = 0x80 + 0x00;
		map_Linestart_DDRAM[1] = 0x80 + 0x40;
		map_Linestart_DDRAM[2] = 0x80 + 0x14;
		map_Linestart_DDRAM[3] = 0x80 + 0x54;
		break;

	case LCD_Type_4x20_KS0073:
		nZeilen = 4;
		nSpalten = 20;
		map_Linestart_DDRAM[0] = 0x80 + 0x00;
		map_Linestart_DDRAM[1] = 0x80 + 0x20;
		map_Linestart_DDRAM[2] = 0x80 + 0x40;
		map_Linestart_DDRAM[3] = 0x80 + 0x60;
		break;

	case LCD_Type_4x20_HD66712:
		nZeilen = 4;
		nSpalten = 20;
		map_Linestart_DDRAM[0] = 0x80 + 0x00;
		map_Linestart_DDRAM[1] = 0x80 + 0x20;
		map_Linestart_DDRAM[2] = 0x80 + 0x40;
		map_Linestart_DDRAM[3] = 0x80 + 0x60;
		break;

	case LCD_Type_4x40:
		nZeilen = 4;
		nSpalten = 40;
		map_Linestart_DDRAM[0] = 0x80 + 0x00;
		map_Linestart_DDRAM[1] = 0x80 + 0x40;
		map_Linestart_DDRAM[2] = 0x80 + 0x00;
		map_Linestart_DDRAM[3] = 0x80 + 0x40;
		break;

	default:
		LCD_State = LCD_State_TypeInvalid;
		return;
		break;

	}

	selectController(0);

// Ports auf Ausgang schalten
// Anm.: die DDR fuer alle Ports sind jeweils mit einer um 1
// niedrigeren Adresse eingeblendet als die PORT...; solange Atmel
// das nicht aendert, kann man ueber (&port_...)[-1] auf das
// DDR-Register zugreifen.
	(&port_d4)[-1] |= (1 << pin_d4);
	(&port_d5)[-1] |= (1 << pin_d5);
	(&port_d6)[-1] |= (1 << pin_d6);
	(&port_d7)[-1] |= (1 << pin_d7);
	(&port_e1)[-1] |= (1 << pin_e1);
	if (&port_e2 != (uint8_t*) 0) {
		(&port_e2)[-1] |= (1 << pin_e2);
	}
	(&port_rs)[-1] |= (1 << pin_rs);

// muss 3 mal hintereinander gesendet werden zur Initialisierung

	_delay_ms(15);

	port_d4 |= (1 << pin_d4);
	port_d5 |= (1 << pin_d5);
	port_d6 &= ~(1 << pin_d6);
	port_d7 &= ~(1 << pin_d7);

	port_rs &= ~(1 << pin_rs); // RS auf 0
	enable();
	if (&port_e2 != (uint8_t*) 0) {
		selectController(1);
		enable();
		selectController(0);
	}

	_delay_ms(5);
	enable();
	if (&port_e2 != (uint8_t*) 0) {
		selectController(1);
		enable();
		selectController(0);
	}

	_delay_ms(1);
	enable();
	if (&port_e2 != (uint8_t*) 0) {
		selectController(1);
		enable();
		selectController(0);
	}
	_delay_ms(1);

// 4 Bit Modus aktivieren
	port_d4 &= ~(1 << pin_d4);
	port_d5 |= (1 << pin_d5);
	port_d6 &= ~(1 << pin_d6);
	port_d7 &= ~(1 << pin_d7);

	enable();
	if (&port_e2 != (uint8_t*) 0) {
		selectController(1);
		enable();
		selectController(0);
	}
	_delay_ms(1);

// 4Bit / 2 Zeilen / 5x7
// TODO: für 1x** ändern?
// TODO: Wieso sind auf dem EA W204B mehr als 7 Pixelzeilen zu sehen?
	writeCommand(0x28);
	if (&port_e2 != (uint8_t*) 0) {
		selectController(1);
		writeCommand(0x28);
		selectController(0);
	}

// Display ein / Cursor aus / kein Blinken
	writeCommand(0x0C);
	if (&port_e2 != (uint8_t*) 0) {
		selectController(1);
		writeCommand(0x0C);
		selectController(0);
	}

// inkrement / kein Scrollen
	writeCommand(0x06);
	if (&port_e2 != (uint8_t*) 0) {
		selectController(1);
		writeCommand(0x06);
		selectController(0);
	}

	clear();
	LCD_State = LCD_State_OK;
} // LCD44780()

// Loescht letzte Fehlernummer:
void LCD44780::resetState() {
	LCD_State = LCD_State_OK;
}

// liefert den Status bzw. den letzten Fehler
LCD44780::e_LCD_State LCD44780::getState(){
	return LCD_State;
}

// liefert aktuelle Zeilennummer
uint8_t LCD44780::getLineNumber() {
	return iZeile;
}

// liefert aktuelle Spaltennummer
uint8_t LCD44780::getColumnNumber() {
	return iSpalte;
}

// Loescht das Display
void LCD44780::clear() {
	if (LCD_Type == LCD_Type_4x40) {
		selectController(0);
	}
	writeCommand(LCD_CLEAR_DISPLAY);
	if (LCD_Type == LCD_Type_4x40) {
		selectController(1);
		writeCommand(LCD_CLEAR_DISPLAY);
		selectController(0);
	}
	_delay_ms(5);
	iZeile = iSpalte = 0;
}

// Sendet den Befehl: Cursor Home
void LCD44780::home() {
	if (LCD_Type == LCD_Type_4x40) {
		selectController(0);
	}
	writeCommand(LCD_CURSOR_HOME);
	_delay_ms(5);
	iZeile = iSpalte = 0;
}

// setzt den Cursor in Zeile y (0..3) Spalte x (0..39)
void LCD44780::setCursor(uint8_t x, uint8_t y) {
	if (x >= nSpalten) {
		LCD_State = LCD_State_ColumnNumberInvalid;
		return;
	}

	if (y >= nZeilen) {
		LCD_State = LCD_State_LineNumberInvalid;
		return;
	}

	if (LCD_Type == LCD_Type_4x40) {
		// Zeilen 0 und 1 werden vom ersten Controller (0) verwurstet,
		// die Zeilen 2 und 3 vom zweiten (1).
		selectController(y >= 2);
	}

	// Sonderbehandlung fuer LCD_Type_1x16_MUX16, wenn x>=8
	if (LCD_Type == LCD_Type_1x16_MUX16 && x >= 8) {
		writeCommand(0x40 + x - 8);
	} else {
		writeCommand(map_Linestart_DDRAM[y] + x);
	}

	iSpalte = x;
	iZeile = y;
}

// Schreibt einen nullterminierten String auf das LCD
void LCD44780::printSZ(const char *p_data) {
	while (p_data && *p_data) {
		printChar(*p_data++);
	}
}

// Schreibt einen String aus dem EEprom auf das LCD
void LCD44780::printSZ_E(const char *p_data) {
	char data;
	while (p_data && (data = (char) eeprom_read_byte(((const uint8_t*) p_data)))) {
		p_data++;
		printChar(data);
	}
}

// Schreibt einen String an der Stelle (x,y) auf das LCD
void LCD44780::printSZ(uint8_t x, uint8_t y, const char *p_data) {
	setCursor(x, y);
	while (*p_data) {
		printChar(*p_data++);
	}
}

// Schreibt einen String aus dem flash auf das LCD
void LCD44780::printSZ_P(uint8_t x, uint8_t y, const char *p_data) {
	char data;
	setCursor(x, y);
	while ((data = (char) pgm_read_byte(((const uint8_t*) p_data)))) {
		p_data++;
		printChar(data);
	}
}

// Schreibt einen String aus dem EEprom auf das LCD
void LCD44780::printSZ_E(uint8_t x, uint8_t y, const char *p_data) {
	char data;
	setCursor(x, y);
	while ((data = (char) eeprom_read_byte(((const uint8_t*) p_data)))) {
		p_data++;
		printChar(data);
	}
}

// sendet ein Datenbyte an das LCD
void LCD44780::printChar(unsigned char data) {
	if (iSpalte < nSpalten) {
		port_rs |= (1 << pin_rs); // RS auf 1 setzen
		writeByte(data);
		++iSpalte;
	} else {
		LCD_State = LCD_State_ColumnNumberInvalid;
	}
}

// setzt Cursor und sendet ein Datenbyte an das LCD
void LCD44780::printChar(uint8_t x, uint8_t y, unsigned char data) {
	setCursor(x, y);
	printChar(data);
}

// definiert ein Zeichen im Zeichensatz an der Position pos (0 bis
// 7) mit den Bitdaten aus arrBitmuster[].
// arrBitmuster[] enthaelt 8 uint8_t, wobei das erste Byte in
// seinen unteren 4 Bit die oberste Zeile des 5x8 Pixel grossen
// Zeichens enthaelt usw. bis arrBitmuster[7], das in seinen
// unteren 5 Bit die unterste Pixelzeile enthaelt.
// Bei einem 2x16 aus dem Pollin Addon-Board kann ich nur die
// ersten 4 Zeichen definieren; bei allen anderen waren es bisher
// immer 8.
void LCD44780::defChar(uint8_t pos, uint8_t arrBitmuster[8]) {
	uint8_t *pBitmuster = arrBitmuster;
	// Kommando set CGRAM address gem. [3]: binaer 01<pos:3><Pixelzeile:3>
	uint8_t cmdSetCGRAMAdress = 0b01000000 | ((pos & 0b11) << 3);

	for (uint8_t iPixelzeile = 0; iPixelzeile < 8; ++iPixelzeile) {
		writeCommand(cmdSetCGRAMAdress++);
		printChar(*pBitmuster++);
		_delay_ms(5);
	}

	// ggf. fuer den anderen Controller auch:
	if (LCD_Type == LCD_Type_4x40) {
		changeController();
		pBitmuster = arrBitmuster;
		cmdSetCGRAMAdress = 0b01000000 | ((pos & 0b11) << 3);
		for (uint8_t iPixelzeile = 0; iPixelzeile < 8; ++iPixelzeile) {
			writeCommand(cmdSetCGRAMAdress++);
			printChar(*pBitmuster++);
		}
		changeController();
	}

	// Mit dem Definieren eines Zeichens scheint zumindest der 2x16
	// vom Pollin Addon-Board seine Ausgabeposition zu vergessen.
	// Neu setzen:
	setCursor(iSpalte, iZeile);
}

// gibt an der aktuellen Cursorposition aus, sonst wie
// std::printf().
// Rueckgabe: Anzahl der ausgegebenen Zeichen (0, wenn es mehr als
// geworden worden waeren, als in die Zeile passt; in diesem Fall
// werden '#' ausgegeben)
uint8_t LCD44780::printf(const char *fmt, ...) {
	uint8_t ret = 0;
	uint8_t restlicheZeilenlaenge = nSpalten - iSpalte;
	char *puffer = (char*) alloca(restlicheZeilenlaenge + 1);
	va_list args;
	va_start(args, fmt);
	ret = vsnprintf(puffer, restlicheZeilenlaenge + 1, fmt, args);
	if (ret > restlicheZeilenlaenge) {
		for (uint8_t i = iSpalte; i < nSpalten; ++i) {
			printChar('#');
		}
		ret = 0;
		LCD_State = LCD_State_ColumnNumberInvalid;
	} else {
		printSZ(puffer);
	}
	va_end(args);
	return ret;
}

// gibt an der aktuellen Cursorposition aus, sonst wie
// std::printf().
// fmt ist ein Zeiger in den Programmspeicher.
// Rueckgabe: Anzahl der ausgegebenen Zeichen (0, wenn es mehr als 40
// geworden worde waeren, als in die Zeile passt; in diesem Fall
// werden '#' ausgegeben)
uint8_t LCD44780::printf_P(const char *fmt, ...) {
	// maximale Laenge fuer ein LCD ist 40, eins fuer die Null:
	uint8_t ret = 0;
	uint8_t restlicheZeilenlaenge = nSpalten - iSpalte;
	char *puffer = (char*) alloca(restlicheZeilenlaenge + 1);
	va_list args;
	va_start(args, fmt);
	ret = vsnprintf_P(puffer, sizeof(puffer), fmt, args);
	if (ret > restlicheZeilenlaenge) {
		for (uint8_t i = iSpalte; i < nSpalten; ++i) {
			printChar('#');
		}
		ret = 0;
		LCD_State = LCD_State_ColumnNumberInvalid;
	} else {
		printSZ(puffer);
	}
	va_end(args);
	return ret;
}

// gibt an der Cursorposition (x,y) n Zeichen aus, sonst wie std::printf().
// Wenn die Ausgabe gem. fmt nicht zu n Zeichen fuehrt, wird rechts mit
// Leerzeichen aufgefuellt.
// Rueckgabe: Anzahl der ausgegebenen Zeichen ohne die aufgefuellten
// Leerzeichen (0, wenn es mehr als n geworden waeren oder wenn
// n>Zeilenlaenge; in diesen Faellen werden '#' ausgegeben)
uint8_t LCD44780::printf(uint8_t x, uint8_t y, uint8_t n, const char *fmt,
		...) {
	uint8_t ret = 0;
	char *puffer = (char *) alloca(n + 1);
	va_list args;
	char fill = '#';

	setCursor(x, y);

	if (n <= (nSpalten - x)) {
		va_start(args, fmt);
		ret = vsnprintf(puffer, n + 1, fmt, args);
		if (ret > n) {
			// Fehler, mehr als n Zeichen wuerden ausgegeben werden
			LCD_State = LCD_State_WidthTooSmall;
			ret = 0;
		} else {
			// Formatierten String ausgeben:
			printSZ(puffer);
			fill = ' ';
		}
		va_end(args);
	} else {
		n = nSpalten - x;
	}

	// Ggf. mit ' ' oder '#' auffuellen:
	for (uint8_t i = ret; i < n; ++i) {
		printChar(fill);
	}
	return ret;
}

// gibt an der Cursorposition (x,y) n Zeichen aus, sonst wie std::printf().
// Wenn die Ausgabe gem. fmt nicht zu n Zeichen fuehrt, wird rechts mit
// Leerzeichen aufgefuellt.
// fmt ist ein Zeiger in den Programmspeicher.
// Rueckgabe: Anzahl der ausgegebenen Zeichen ohne die aufgefuellten
// Leerzeichen (0, wenn es mehr als n geworden waeren oder wenn
// n>Zeilenlaenge; in diesen Faellen werden '#' ausgegeben)
int LCD44780::printf_P(uint8_t x, uint8_t y, uint8_t n, const char *fmt, ...) {
	// maximale Laenge fuer ein LCD ist 40, eins fuer die Null:
	int ret = 0;
	char *puffer = (char *) alloca(n + 1);
	va_list args;
	char fill = '#';

	setCursor(x, y);

	if (n <= (nSpalten - x)) {
		va_start(args, fmt);
		ret = vsnprintf_P(puffer, n + 1, fmt, args);
		if (ret > n) {
			// Fehler, mehr als n Zeichen wuerden ausgegeben werden
			LCD_State = LCD_State_WidthTooSmall;
			ret = 0;
		} else {
			// Formatierten String ausgeben:
			printSZ(puffer);
			fill = ' ';
		}
		va_end(args);
	} else {
		n = nSpalten - x;
		;
	}

	// Ggf. mit ' ' oder '#' auffuellen:
	for (uint8_t i = ret; i < n; ++i) {
		printChar(fill);
	}
	return ret;
}

LCD44780::~LCD44780() {
	// TODO Auto-generated destructor stub
}

// waehlt den ersten (0) oder zweiten (1) Controller aus
void LCD44780::selectController(uint8_t iCtrlr) {
	switch (iCtrlr) {
	case 0:
		p_port_e = &port_e1;
		p_pin_e = &pin_e1;
		break;

	case 1:
		p_port_e = &port_e2;
		p_pin_e = &pin_e2;
		break;

	default:

		break;
	}
}

// wechselt den Controller (1 , wenn vorher 0; 0 wenn vorher 1)
void LCD44780::changeController() {
	if (p_port_e == &port_e2 && p_pin_e == &pin_e2) {
		p_port_e = &port_e1;
		p_pin_e = &pin_e1;
	} else {
		p_port_e = &port_e2;
		p_pin_e = &pin_e2;
	}
}

// erzeugt den Enable-Puls fuer den Controller, der zuletzt mit
// selectController() ausgewaehlt wurde
void LCD44780::enable() {
	// Bei Problemen ggf. Pause gemaess Datenblatt des LCD Controllers einfuegen

	// http://www.mikrocontroller.net/topic/81974#685882
	*p_port_e |= (1 << *p_pin_e);

	// Bei Problemen ggf. Pause gemaess Datenblatt des LCD Controllers verlaengern
	_delay_us(1); // kurze Pause

	// http://www.mikrocontroller.net/topic/80900
	*p_port_e &= ~(1 << *p_pin_e);
}

// sendet ein Daten- oder Kommandobyte (RS am LCD muss vorher
// entsprechend gesetzt sein)
void LCD44780::writeByte(unsigned char data) {
	// oberes Nibble auf den LCD-Pins 4..7 ausgeben:

	if (data & 0x10) {
		port_d4 |= (1 << pin_d4);
	} else {
		port_d4 &= ~(1 << pin_d4);
	}
	if (data & 0x20) {
		port_d5 |= (1 << pin_d5);
	} else {
		port_d5 &= ~(1 << pin_d5);
	}
	if (data & 0x40) {
		port_d6 |= (1 << pin_d6);
	} else {
		port_d6 &= ~(1 << pin_d6);
	}
	if (data & 0x80) {
		port_d7 |= (1 << pin_d7);
	} else {
		port_d7 &= ~(1 << pin_d7);
	}
	enable();

	// unteres Nibble auf den LCD-Pins 4..7 ausgeben:

	if (data & 0x01) {
		port_d4 |= (1 << pin_d4); // setzen
	} else {
		port_d4 &= ~(1 << pin_d4);
	}
	if (data & 0x02) {
		port_d5 |= (1 << pin_d5);
	} else {
		port_d5 &= ~(1 << pin_d5);
	}
	if (data & 0x04) {
		port_d6 |= (1 << pin_d6);
	} else {
		port_d6 &= ~(1 << pin_d6);
	}
	if (data & 0x08) {
		port_d7 |= (1 << pin_d7);
	} else {
		port_d7 &= ~(1 << pin_d7);
	}
	enable();

	// Dem LCD Zeit geben, das Kommando auszufuehren
	_delay_us(42);
}

// sendet einen Befehl an das LCD
void LCD44780::writeCommand(unsigned char command) {
	port_rs &= ~(1 << pin_rs); // RS auf 0 setzen
	writeByte(command);
}

